; _______________________________________________________________________________________
;|                                                                                       |
;| ..::FreshLib::..  Free, open source. Licensed under "BSD 2-clause" license."          |
;|_______________________________________________________________________________________|
;
;  Description: Context is a object that represents drawing surface in FreshLib
;
;  Target OS: Win32
;
;  Dependencies:
;
;  Notes:
;_________________________________________________________________________________________

uses user32, gdi32

; Some of these constants are probably wrong!
cmClear       =     R2_BLACK
cmAnd         =     R2_MASKPEN
cmAndReverse  =     R2_NOTMASKPEN
cmCopy        =     R2_COPYPEN
cmAndInverted =     R2_MASKPENNOT
cmNoOp        =     R2_NOP
cmXor         =     R2_XORPEN
cmOr          =     R2_MERGEPEN
cmNor         =     R2_NOTMERGEPEN
cmEquiv       =     R2_NOTXORPEN
cmInvert      =     R2_NOT
cmXorReverse  =     R2_NOTXORPEN
cmCopyInverted =    R2_NOTCOPYPEN
cmOrInverted   =    R2_NOTMERGEPEN
cmNand         =    R2_NOTMASKPEN
cmSet          =    R2_WHITE


proc AllocateContext, .raster
begin
        push    ebx ecx edx

        stdcall GetMem, sizeof.TContext
        mov     ebx, eax

        push    [.raster]
        pop     [ebx+TContext.raster]

        invoke  IsWindow, [.raster]
        test    eax, eax
        jz      .not_window

        invoke  GetDC, [.raster]
        mov     [ebx+TContext.handle], eax
        test    eax, eax
        jz      .error

        invoke  SaveDC, [ebx+TContext.handle]

.exit_ok:
        clc
        mov     eax, ebx
        pop     edx ecx ebx
        return

.not_window:    ; then it must be handle to image bitmap...
        invoke  GetObjectType, [.raster]
        cmp     eax, OBJ_BITMAP
        jne     .error

        invoke  CreateCompatibleDC, 0
        mov     [ebx+TContext.handle], eax
        test    eax, eax
        jz      .error

        invoke  SaveDC, ebx
        invoke  SelectObject, [ebx+TContext.handle], [.raster]
        jmp     .exit_ok

.error:
        stc
        stdcall FreeMem, ebx
        xor     eax, eax
        pop     edx ecx ebx
        return
endp




proc ReleaseContext, .context
begin
        push    eax ebx ecx edx

        mov     ebx, [.context]

; delete all selected objects...
        invoke  GetCurrentObject, [ebx+TContext.handle], OBJ_PEN
        push    eax
        invoke  GetCurrentObject, [ebx+TContext.handle], OBJ_BRUSH
        push    eax
        invoke  GetCurrentObject, [ebx+TContext.handle], OBJ_FONT
        push    eax

        invoke  RestoreDC, [ebx+TContext.handle], -1

        invoke  DeleteObject ; from the stack
        invoke  DeleteObject ; from the stack
        invoke  DeleteObject ; from the stack

        invoke  GetObjectType, [ebx+TContext.handle]
        cmp     eax, OBJ_MEMDC
        je      .delete

        cmp     eax, OBJ_DC
        je      .release

        stc
.exit:
        pop     edx ecx ebx eax
        return

.delete:
        invoke  DeleteDC, [ebx+TContext.handle]
        jmp     .free

.release:
        invoke  ReleaseDC, [ebx+TContext.raster] , [ebx+TContext.handle]

.free:
        stdcall FreeMem, ebx
        clc
        jmp     .exit
endp



proc  SetDrawMode, .context, .mode
begin
        push    eax ebx ecx edx
        mov     ebx, [.context]
        test    ebx, ebx
        jz      .finish

        push    [.mode]
        pop     [ebx+TContext.drawmode]
        invoke  SetROP2, [ebx+TContext.handle], [.mode]

.finish:
        pop     edx ecx ebx eax
        return
endp




proc SetLineStyle, .context, .pLineStyle
begin
        push    eax ebx ecx edx esi

        mov     ebx, [.pLineStyle]
        mov     esi, [.context]

        mov     eax, [ebx+TLineStyle.color]
        bswap   eax                             ; FreshLib uses Linux type color - red in the MSB
        ror     eax, 8

        mov     edx, PS_SOLID
        cmp     [ebx+TLineStyle.DashCount], 0
        je      @f
        mov     edx, PS_DOT
        mov     [ebx+TLineStyle.width], 0
@@:
        invoke  CreatePen, edx, [ebx+TLineStyle.width], eax     ;[ebx+TLineStyle.color]
        invoke  SelectObject, [esi+TContext.handle], eax
        invoke  DeleteObject, eax

        cmp     [ebx+TLineStyle.DashCount], 0
        je      .exit

        mov     eax, [ebx+TLineStyle.clSpace]
        bswap   eax                             ; FreshLib uses Linux type color - red in the MSB
        ror     eax, 8

        invoke  SetBkColor, [esi+TContext.handle], eax
        invoke  SetBkMode, [esi+TContext.handle], OPAQUE


.exit:
        pop     esi edx ecx ebx eax
        return
endp






proc SetClipRectangle, .context, .pBounds
begin
        push    eax ebx ecx edx

        xor     ebx, ebx
        mov     edx, [.pBounds]
        test    edx, edx
        jz      .setit

        mov     eax, [edx+TBounds.width]
        mov     ecx, [edx+TBounds.height]
        add     eax, [edx+TBounds.x]
        add     ecx, [edx+TBounds.y]
        invoke  CreateRectRgn, [edx+TBounds.x], [edx+TBounds.y], eax, ecx
        mov     ebx, eax

.setit:
        mov     edx, [.context]
        invoke  SelectClipRgn, [edx+TContext.handle], ebx

        test    ebx, ebx
        jz      .exit
        invoke  DeleteObject, ebx
.exit:
        pop     edx ecx ebx eax
        return
endp




;proc SetLineStyle, .context, .pLineStyle
;.brush LOGBRUSH
;.width dd ?
;.dashcount dd ?
;.dashes rd 16
;begin
;        push    eax ebx ecx edx esi
;
;        mov     ebx, [.pLineStyle]
;        mov     esi, [.context]
;
;        mov     ecx, [ebx+TLineStyle.DashCount]
;        cmp     ecx, 16
;        jbe     @f
;        mov     ecx, 16
;@@:
;        mov     [.dashcount], ecx
;
;.dashloop:
;        dec     ecx
;        js      .enddash
;        movzx   eax, byte [ebx+TLineStyle.DashItems+ecx]
;        mov     [.dashes+4*ecx], eax
;        jmp     .dashloop
;
;.enddash:
;        push    [ebx+TLineStyle.width]
;        pop     [.width]
;
;        mov     [.brush.lbStyle], BS_SOLID
;        mov     eax, [ebx+TLineStyle.color]
;        bswap   eax                             ; FreshLib uses Linux type color - red in the MSB
;        ror     eax, 8
;
;        mov     [.brush.lbColor], eax
;
;        mov     edx, PS_SOLID
;        xor     ecx, ecx
;        cmp     [.dashcount], 0
;        je      @f
;        mov     edx, PS_USERSTYLE
;        lea     ecx, [.dashes]
;@@:
;
;        or      edx, PS_GEOMETRIC
;        cmp     [.width], 0
;        jne     @f
;        and     edx, not PS_GEOMETRIC
;        inc     [.width]
;@@:
;        or      edx, [ebx+TLineStyle.flags]
;        lea     eax, [.brush]
;
;        invoke  ExtCreatePen, edx, [.width], eax, [.dashcount], ecx
;        invoke  SelectObject, [esi+TContext.handle], eax
;
;
;        invoke  SetBkMode, [esi+TContext.handle], OPAQUE
;
;        mov     eax, [ebx+TLineStyle.color]
;        cmp     [ebx+TLineStyle.DashCount], 0
;        je      @f
;        mov     eax, [ebx+TLineStyle.clSpace]
;@@:
;        bswap   eax                             ; FreshLib uses Linux type color - red in the MSB
;        ror     eax, 8
;
;        invoke  SetBkColor, [esi+TContext.handle], eax
;
;        pop     esi edx ecx ebx eax
;        return
;endp
;


