; _______________________________________________________________________________________
;|                                                                                       |
;| ..::FreshLib::..  Free, open source. Licensed under "BSD 2-clause" license."          |
;|_______________________________________________________________________________________|
;
;  Description: Context is a object that represents drawing surface in FreshLib
;
;  Target OS: Any
;
;  Dependencies:
;
;  Notes: This file contains OS independent part and includes OS dependent files.
;_________________________________________________________________________________________
module "Graphic context library"

struct TBounds
  .x      dd ?
  .y      dd ?
  .width  dd ?
  .height dd ?
ends



struct TContext
  .raster     dd ?          ; handle to raster object (depends on TargetOS)
  .handle     dd ?          ; handle to context object (depends on TargetOS)
  .clipregion dd ?          ; clipping region.
  .drawmode   dd ?
ends


struct TDashListItem
  .dash  dd ?
  .space dd ?
ends


struct TLineStyle
  .width dd ?                   ; width of the line in pixels. Width 0 is 1px width. It have special meaning in Linux for example.
  .color dd ?                   ; color of the line in format (from LSB to MSB) bb, gg, rr, aa ; Use of alpha value is something not very clear for all implementations...
  .flags dd ?                   ; set of lsEndCapXXX; lsJoinXXX
  .clSpace dd ?                 ; space color.
  .DashCount dd ?               ; count of dash items in the dash list. For solid lines DashCount=0
  .DashItems:                   ; arbitrary count of TDashListItem
ends


; Creates static TLineStyle structure in design time.

struc LineStyle width*, color*, flags, spacecolor, [dash, space] {
common
local ..count, ..dashcount
  ..count = 0

common
  dd width
  dd color
  if flags eq
    dd lsEndCapRound or lsJoinRound
  else
    dd flags
  end if

  if spacecolor eq
    dd 0
  else
    dd spacecolor
  end if

  dd ..dashcount

forward
  if ~dash eq
    db dash, space
    ..count = ..count + 2
  end if

common
  ..dashcount = ..count
}



proc SetSimpleLine, .context, .color
.style TLineStyle
       rd 16
begin
        push    eax

        lea     eax, [.style]
        mov     [eax+TLineStyle.width], 0
        mov     [eax+TLineStyle.flags], 0
        mov     [eax+TLineStyle.DashCount], 0
        mov     [eax+TLineStyle.clSpace],0

        push    [.color]
        pop     [eax+TLineStyle.color]


        stdcall SetLineStyle, [.context], eax
        pop     eax
        return
endp



include '%TargetOS%/context.asm'

endmodule