; _______________________________________________________________________________________
;|                                                                                       |
;| ..::FreshLib::..  Free, open source. Licensed under "BSD 2-clause" license."          |
;|_______________________________________________________________________________________|
;
;  Description: Actions management library.
;
;  Target OS: Any
;
;  Dependencies:
;
;  Notes: "Action" is set of executable code, combined with user interface attributes -
;         caption, accelerator keys, enable/disable status flag etc.
;         Different UI elements, such as buttons, menu items etc. can be attached to
;         the action object and to use its attributes. When the programmer changes
;         something in the action, all UI elements are updated automatically.
;_________________________________________________________________________________________

struc TAction caption, hint, icon, FastKey, enabled, OnExecute, OnIdle {
common
  .Text      dd  caption    ; handle or pointer to the caption string.
  .Hint      dd  hint       ; handle or pointer to the Hint text.
  .icon      dd  icon       ; pointer to TImage
  .FastKey   dd  FastKey    ; pointer to keyboard accelerator combination.
  .Enabled   dd  enabled    ; if FALSE, the action is not active.
  .OnExecute dd  OnExecute  ; pointer to procedure to be executed on activation of the action.
  .OnIdle    dd  OnIdle     ; pointer to procedure that to be executed once, when there is no other events in the queue.
}

virtual at 0
  TAction TAction ?, ?, ?, ?, ?, ?, ?
end virtual


struc TFastKey flags, key, action {
  .flags  dd flags
  .key    dd key
  .action dd action
}
virtual at 0
  TFastKey TFastKey ?, ?, ?
end virtual


struct TActionList
  .count dd ?
  .accelerators dd ?
  .actions:
ends



macro ActionList name, [actname, caption, hint, icon, FastKey, OnExecute, OnIdle] {
common
  local i, count, acclist
  i = 0
  label name dword
  .count dd count
  .accelerators dd acclist
forward
  local .accitem, txtlbl, hintlbl

  txtlbl text caption
  hintlbl text hint

  actname TAction txtlbl, hintlbl, icon, accitem, 1, OnExecute, OnIdle

  i = i + 1
common
  count = i
  label acclist dword

forward
  if FastKey eq NONE
    accitem = 0
  else
    .accitem TFastKey FastKey, name#actname
  end if
}