; _______________________________________________________________________________________
;|                                                                                       |
;| ..::FreshLib::..  Free, open source. Licensed under "BSD 2-clause" license."          |
;|_______________________________________________________________________________________|
;
;  Description: TButton object class
;
;  Target OS: Any
;
;  Dependencies:
;
;  Notes: Represents GUI button.
;_________________________________________________________________________________________
module "TButton library"

ObjectClass Button,                     \
            Window,                     \
            0,                          \
            0,                          \
            TButton.Get,                \
            TButton.Set,                \
            0,                          \
            TButton.SysEventHandler


; Button states for the field TButton.state
btnNormal    = 0
btnPressed   = 1
btnPointed   = 2
btnChecked   = 3




object TButton, TWindow
  .state      dd ?
  .Ficon      dd ?
  .FiconAlign dd ?
  .Ftextalign dd ?

  .ModalResult dd ?
  .OnClick dd ?

  param .TextAlign
  param .Icon
  param .IconPosition
endobj



proc TButton.Get, .obj, .paramID
begin
        mov     eax, [.obj]

        stdcall IsObject, eax, CButton
        jc      .exit

        cmp     [.paramID], TButton.TextAlign
        je      .gettextalign

        cmp     [.paramID], TButton.IconPosition
        je      .geticonposition

        cmp     [.paramID], TButton.Icon
        je      .geticon

        stc
.exit:
        return

.geticonposition:
        mov     eax, [eax+TButton.FiconAlign]
        clc
        return

.geticon:
        mov     eax, [eax+TButton.Ficon]
        clc
        return

.gettextalign:
        mov     eax, [eax+TButton.Ftextalign]
        clc
        return
endp





dproc TButton.Set, .obj, .paramID, .value
begin
        push    eax ecx
        mov     ecx, [.obj]

dispatch [.paramID]

        stc
        pop     ecx eax
        return

.refresh:
        execute [.obj], TWindow.Refresh
        clc
        pop     ecx eax
        return


oncase TButton.TextAlign
        push    [.value]
        pop     [ecx+TButton.Ftextalign]
        jmp     .refresh


oncase TButton.Icon
        push    [.value]
        pop     [ecx+TButton.Ficon]
        jmp     .refresh


oncase TButton.IconPosition
        push    [.value]
        pop     [ecx+TButton.FiconAlign]
        jmp     .refresh

enddp




dproc TButton.SysEventHandler, .pObj, .pEvent
begin
        pushad

        mov     esi, [.pObj]
        mov     ebx, [.pEvent]

        dispatch [ebx+TSysEvent.event]

.finish:
        stc
        popad
        return


;. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .

oncase sePaint
; PAINT event is very important.
        mov     edx, bxFlat
        mov     edi, [clButtonBk]       ; background
        mov     ecx, [clButtonTxt]      ; text

        cmp     [esi+TButton.state], btnNormal
        je      .drawit

        mov     edx, bxRaised
        mov     edi, [clButtonHotBk]
        mov     ecx, [clButtonHotTxt]

        cmp     [esi+TButton.state], btnPointed
        je      .drawit

        mov     edx, bxSunken
        mov     edi, [clButtonBk]
        mov     ecx, [clButtonHotTxt]

        cmp     [esi+TButton.state], btnPressed
        je      .drawit

        mov     edx, bxFlat

.drawit:
locals
 .bounds TBounds
 .clTxt dd ?
endl
        mov     [.clTxt], ecx

        mov     ecx, [esi+TButton._width]
        mov     eax, [esi+TButton._height]
        mov     [.bounds.x], 0
        mov     [.bounds.y], 0
        mov     [.bounds.width], ecx
        mov     [.bounds.height], eax
        lea     eax, [.bounds]
        stdcall [DrawBox], [ebx+TPaintEvent.context], eax, edi, edx

; first draw the icon and then resize bounds for the text.
        mov     edi, [esi+TButton.Ficon]
        test    edi, edi
        jz      .draw_text

        cmp     [esi+TButton.FiconAlign], AlignLeft
        jne     @f

; align left
        mov     eax, [.bounds.x]
        mov     ecx, [.bounds.height]
        add     eax, 2
        sub     ecx, [edi+TImage.height]
        sar     ecx, 1
        add     ecx, [.bounds.y]

        mov     edx, [edi+TImage.width]
        add     edx, 4
        add     [.bounds.x], edx
        sub     [.bounds.width], edx
        jmp     .draw_icon

@@:
        cmp     [esi+TButton.FiconAlign], AlignRight
        jne     @f

; align right
        mov     edx, [edi+TImage.width]

        mov     eax, [.bounds.x]
        add     eax, [.bounds.width]
        sub     eax, edx
        sub     eax, 2
        sub     [.bounds.width], edx
        sub     [.bounds.width], 4

        mov     ecx, [.bounds.height]
        sub     ecx, [edi+TImage.height]
        sar     ecx, 1
        add     ecx, [.bounds.y]
        jmp     .draw_icon

@@:
        cmp     [esi+TButton.FiconAlign], AlignTop
        jne     @f

; align top
        mov     ecx, [.bounds.y]
        add     ecx, 2
        mov     eax, [.bounds.width]
        sub     eax, [edi+TImage.width]
        sar     eax, 1
        add     eax, [.bounds.x]

        mov     edx, [edi+TImage.height]
        add     edx, 4
        add     [.bounds.y], edx
        sub     [.bounds.height], edx
        jmp     .draw_icon

@@:
        cmp     [esi+TButton.FiconAlign], AlignBottom
        jne     .draw_text

; align bottom
        mov     edx, [edi+TImage.height]

        mov     ecx, [.bounds.y]
        add     ecx, [.bounds.height]
        sub     ecx, edx
        sub     ecx, 2
        sub     [.bounds.height], edx
        sub     [.bounds.height], 4

        mov     eax, [.bounds.width]
        sub     eax, [edi+TImage.width]
        sar     eax, 1
        add     eax, [.bounds.x]

.draw_icon:
        stdcall DrawImage, [ebx+TPaintEvent.context], [esi+TButton.Ficon], eax, ecx

.draw_text:
        stdcall Get, esi, TWindow.Caption
        test    eax, eax
        jz      .finish

        push    eax eax
        stdcall StrLen, eax
        mov     ecx, eax
        pop     eax
        lea     edx, [.bounds]
        stdcall DrawTextBox, [ebx+TPaintEvent.context], eax, edx, [esi+TButton.Ftextalign], 0, [.clTxt]
        stdcall StrDel ; from the stack
        jmp     .finish


;. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .

oncase seMouseEnter

        mov     [esi+TButton.state], btnPointed
        execute esi, TButton.Refresh
        jmp     .finish


;. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .


oncase seMouseLeave
        mov     [esi+TButton.state], btnNormal
        execute esi, TButton.Refresh
        jmp     .finish


;. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .


oncase seMouseBtnPress
        mov     [esi+TButton.state], btnPressed
        execute esi, TWindow.Refresh
        jmp     .finish


;. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .


oncase seMouseBtnRelease

        cmp     [esi+TButton.state], btnPressed
        jne     .finish

        mov     [esi+TButton.state], btnPointed
        execute esi, TButton.Refresh

        cmp     [esi+TButton.ModalResult], mrNone
        je      @f

        stdcall Get, esi, TButton.Parent
        stdcall IsObject, eax, CForm
        jc      @f

        push    [esi+TButton.ModalResult]
        pop     [eax+TForm.ModalResult]

@@:
        cmp     [esi+TButton.OnClick], 0
        je      .finish

        stdcall [esi+TButton.OnClick], esi, [ebx+TMouseButtonEvent.Button]
        jmp     .finish
enddp


endmodule