; _______________________________________________________________________________________
;|                                                                                       |
;| ..::FreshLib::..  Free, open source. Licensed under "BSD 2-clause" license."          |
;|_______________________________________________________________________________________|
;
;  Description: TEdit object class
;
;  Target OS: Any
;
;  Dependencies:
;
;  Notes: Represents single line edit control.
;_________________________________________________________________________________________
module "TEdit library"

ObjectClass Edit,                       \
            Window,                     \
            TEdit.Create,               \
            TEdit.Destroy,              \
            TEdit.Get,                  \
            TEdit.Set,                  \
            0,                          \
            TEdit.SysEventHandler


object TEdit, TWindow
  ._Text  dd ?
  ._Len   dd ?  ; the length of the string in characters. (UTF-8)
  ._Start dd ?  ; Where the string begins to be displayed in the edit window.
  ._Pos   dd ?  ; Position of the caret in the string.
  ._Sel   dd ?  ; Position of the selction in the string.

  ._MarginLeft dd ?
  ._MarginRight dd ?

  ._Focused dd ? ; flag

  param .Text
endobj


proc TEdit.Create, .obj
begin
        push    ecx eax
        stdcall StrNew
        mov     ecx, [.obj]
        mov     [ecx+TEdit._Text], eax

; for test. Normal value is 0 for these fields.
;        mov     [ecx+TEdit._MarginLeft], 8
;        mov     [ecx+TEdit._MarginRight], 16

        mov     [ecx+TEdit._cursor], mcText

        pop     eax ecx
        clc
        return
endp


proc TEdit.Destroy, .obj
begin
        mov     eax, [.obj]
        stdcall StrDel, [eax+TEdit._Text]
        return
endp


proc TEdit.Get, .obj, .paramID
begin
        stdcall IsObject, [.obj], CEdit
        jc      .exit

        cmp     [.paramID], TEdit.Text
        je      .gettext

        stc
.exit:
        return

.gettext:
        mov     eax, [.obj]
        stdcall StrDup, [eax+TEdit._Text]
        clc
        return
endp




proc TEdit.Set, .obj, .paramID, .value
begin
        push    esi

        mov     esi, [.obj]

        cmp     [.paramID], TEdit.Text
        je      .settext

        stc
        pop     esi
        return

.settext:
        push    eax
        mov     [esi+TEdit._Start], 0
        mov     [esi+TEdit._Pos], 0
        mov     [esi+TEdit._Sel], 0

        lea     eax, [esi+TEdit._Text]
        stdcall SetString, eax, [.value]
        stdcall StrLenUtf8, [esi+TEdit._Text], -1
        mov     [esi+TEdit._Len], eax

        execute esi, TEdit.Refresh
        clc
        pop     eax
        pop     esi
        return
endp






proc TEdit.SysEventHandler, .pObj, .pEvent
.changes dd ?
.prevpos dd ?
.PosChanged  = 1
.NeedRefresh = 2
.ShowCaret   = 4
begin
        push    eax ebx ecx edx esi edi

        mov     [.changes], 0
        mov     esi, [.pObj]
        mov     ebx, [.pEvent]
        mov     eax, [ebx+TSysEvent.event]

        cmp     eax, sePaint
        je      .onpaint

        cmp     eax, seKbdKeyPress
        je      .keypress

        cmp     eax, seFocusIn
        je      .focusin

        cmp     eax, seFocusOut
        je      .focusout

.finish:
        stc
        pop     edi esi edx ecx ebx eax
        return

;............................................................................................

.focusin:
        mov     [esi+TEdit._Focused], TRUE
        stdcall CaretAttach, esi
        or      [.changes], .PosChanged or .ShowCaret
        jmp     .endmove

.focusout:
        stdcall CaretShow, FALSE
        mov     [esi+TEdit._Focused], FALSE
        or      [.changes], .PosChanged
        jmp     .endmove

;............................................................................................

.keypress:
;        mov     eax, [ebx+TKeyboardEvent.kbdStatus]
;        OutputRegister regEAX, 16

        mov     eax, [esi+TEdit._Pos]
        mov     [.prevpos], eax

        mov     eax, [ebx+TKeyboardEvent.key]
        test    eax, eax
        jz      .no_char

        cmp     eax, $20
        jb      .no_char

        cmp     eax, $7f
        je      .no_char

        push    eax

        call    .ClearSelection

        stdcall StrPtr, [esi+TEdit._Text]
        mov     ecx, eax
        stdcall StrOffsUtf8, eax, [esi+TEdit._Pos]
        sub     ecx, eax
        neg     ecx

        pop     eax
        stdcall StrCharInsert, [esi+TEdit._Text], eax, ecx
        inc     [esi+TEdit._Len]

        mov     [ebx+TKeyboardEvent.kbdStatus], 0
        or      [.changes], .NeedRefresh
        jmp     .right

.no_char:
        mov     eax, [ebx+TKeyboardEvent.scancode]

        cmp     eax, keyLeftNumpad
        je      .left
        cmp     eax, keyLeft
        je      .left

        cmp     eax, keyRightNumpad
        je      .right
        cmp     eax, keyRight
        je      .right

        cmp     eax, keyHomeNumpad
        je      .home
        cmp     eax, keyHome
        je      .home

        cmp     eax, keyEndNumpad
        je      .end
        cmp     eax, keyEnd
        je      .end

        cmp     eax, keyDelNumpad
        je      .del
        cmp     eax, keyDelete
        je      .del

        cmp     eax, keyBackSpace
        je      .backdel

        jmp     .finish

.left:
        cmp     [esi+TEdit._Pos], 0
        jle     .endmove

        or      [.changes], .PosChanged
        dec     [esi+TEdit._Pos]
        jmp     .endmove

.right:
        mov     eax, [esi+TEdit._Pos]
        cmp     eax, [esi+TEdit._Len]
        jae     .finish

        or      [.changes], .PosChanged
        inc     [esi+TEdit._Pos]
        jmp     .endmove

.home:
        mov     [esi+TEdit._Pos], 0
        mov     [esi+TEdit._Start], 0
        mov     [.changes], .PosChanged or .NeedRefresh
        jmp     .endmove

.end:
        mov     eax, [esi+TEdit._Len]
        mov     [esi+TEdit._Pos], eax
        or      [.changes], .PosChanged
        jmp     .endmove

.backdel:
        call    .ClearSelection
        jnc     .endmove

        cmp     [esi+TEdit._Pos], 0
        jle     .endmove

        or      [.changes], .PosChanged
        dec     [esi+TEdit._Pos]

.del:
        call    .ClearSelection
        jnc     .endmove

        mov     eax, [esi+TEdit._Pos]
        cmp     eax, [esi+TEdit._Len]
        jae     .finish                ; can't delete after the end of string.

        stdcall StrPtr, [esi+TEdit._Text]

        mov     ecx, eax
        stdcall StrOffsUtf8, eax, [esi+TEdit._Pos]
        sub     eax, ecx

        stdcall StrSplit, [esi+TEdit._Text], eax
        mov     ebx, eax

        stdcall StrPtr, ebx
        stdcall DecodeUtf8, [eax]

        stdcall StrCopyPart, ebx, ebx, edx, -1
        stdcall StrCat, [esi+TEdit._Text], ebx
        stdcall StrDel, ebx

        dec     [esi+TEdit._Len]
        or      [.changes], .NeedRefresh

.endmove:
        mov     ebx, [.pEvent]
        mov     eax, [esi+TEdit._Pos]
        cmp     [ebx+TSysEvent.event], seKbdKeyPress
        jne     .setselection

        test    [ebx+TKeyboardEvent.kbdStatus], maskShift
        jnz     .refresh

.setselection:
        mov     ecx, [.prevpos]
        cmp     ecx, [esi+TEdit._Sel]
        je      @f
        or      [.changes], .NeedRefresh
@@:
        mov     [esi+TEdit._Sel], eax

        test    [.changes], .NeedRefresh
        jz      .caretmove

.refresh:
        execute [.pObj], TEdit.Refresh

.caretmove:
        cmp     [esi+TEdit._Focused], 0
        je      .finish

        test    [.changes], .PosChanged
        jz      .showcaret

        mov     eax, [esi+TEdit._Pos]
        cmp     eax, [esi+TEdit._Start]
        jl      .scroll

        stdcall StrPtr, [esi+TEdit._Text]
        mov     edi, eax

        stdcall StrOffsUtf8, edi, [esi+TEdit._Pos]
        mov     ecx, eax

        stdcall StrOffsUtf8, edi, [esi+TEdit._Start]
        sub     ecx, eax                           ; offset to char under cursor.
        jns     .leftok

.scroll:
        mov     eax, [esi+TEdit._Start]
        sub     eax, 8
        test    eax, eax
        jns         @f
        xor     eax, eax
@@:
        mov     [esi+TEdit._Start], eax
        mov     [.changes], .PosChanged or .NeedRefresh
        jmp     .endmove

.leftok:
        mov     edi, eax                           ; pointer to start of the text.

        stdcall AllocateContext, [esi+TEdit.handle]
        push    eax

        stdcall GetTextBounds, eax, edi, ecx, 0
        lea     ecx, [eax+2]       ; x coordinate of the cursor.
        add     ecx, [esi+TEdit._MarginLeft]

        stdcall ReleaseContext ; from the stack.

        mov     eax, [esi+TEdit._width]
        sub     eax, [esi+TEdit._MarginRight]
        sub     eax, ecx
        cmp     eax, 2
        jge     .caretok

        mov     eax, [esi+TEdit._Start]
        add     eax, 8
        cmp     eax, [esi+TEdit._Len]
        jle     @f
        mov     eax, [esi+TEdit._Len]
@@:
        mov     [esi+TEdit._Start], eax

        mov     [.changes], .PosChanged or .NeedRefresh
        jmp     .endmove


.caretok:
        mov     edx, [esi+TEdit._height]
        sub     edx, 4

        stdcall CaretChange, ecx, 2, 1, edx

.showcaret:
        test    [.changes], .ShowCaret
        jz      .finish

        stdcall CaretShow, TRUE
        jmp     .finish

;............................................................................................

.onpaint:
locals
 .bounds TBounds
 .selbeg dd ?
 .selend dd ?
 .ypos   dd ?
endl
        mov     eax, [esi+TEdit._Pos]
        mov     ecx, [esi+TEdit._Sel]
        cmp     eax,ecx
        jle     @f
        xchg    eax, ecx
@@:
        mov     [.selbeg], eax
        mov     [.selend], ecx

        mov     ecx, [esi+TEdit._width]
        mov     eax, [esi+TEdit._height]

        mov     [.bounds.x], 0
        mov     [.bounds.y], 0
        mov     [.bounds.width], ecx
        mov     [.bounds.height], eax

        lea     eax, [.bounds]
        stdcall [DrawBox], [ebx+TPaintEvent.context], eax, [clEditBk], bxSunken

        mov     eax, [esi+TEdit._MarginLeft]
        mov     ecx, [esi+TEdit._MarginRight]
        add     [.bounds.x], eax
        sub     [.bounds.width], eax
        sub     [.bounds.width], ecx
        cmp     [.bounds.width], 0
        jle     .finish

        mov     ecx, [esi+TEdit._Start]

        stdcall StrOffsUtf8, [esi+TEdit._Text], ecx
        mov     edi, eax

        mov     eax, [.bounds.y]
        add     eax, [.bounds.height]
        sub     eax, 4
        mov     [.ypos], eax   ; y position of the characters.

.drawloop:
        stdcall DecodeUtf8, [edi]
        jc      .finish

        test    eax, eax
        jz      .eraseremaining

        mov     eax, [clEditTxt]
        cmp     ecx, [.selbeg]
        jl      .colorok
        cmp     ecx, [.selend]
        jge     .colorok
        mov     eax, [clEditSelTxt]
.colorok:
        push    eax  0 [.ypos] [.bounds.x] edx edi

        push    0 edx edi
        add     edi, edx
        stdcall GetTextBounds, [ebx+TPaintEvent.context] ; remaining from the stack

        sub     [.bounds.width], eax
        js      .nodraw

        cmp     ecx, [.selbeg]
        jl      .drawtxt
        cmp     ecx, [.selend]
        jge     .drawtxt

        mov     esi, [.ypos]
        sub     esi, edx
        add     edx, 2
        stdcall DrawFillRect, [ebx+TPaintEvent.context], [.bounds.x], [.bounds.y], eax, [.bounds.height], [clEditSel]

.drawtxt:
        add     [.bounds.x], eax
        stdcall DrawString, [ebx+TPaintEvent.context] ; remaining from the stack

        inc     ecx
        jmp     .drawloop

.nodraw:
        add     esp, 24 ; 6*4
        jmp     .finish

.eraseremaining:
        stdcall DrawFillRect, [ebx+TPaintEvent.context], [.bounds.x], [.bounds.y], [.bounds.width], [.bounds.height], [clEditBk]
        jmp     .finish

.ClearSelection:
        mov     eax, [esi+TEdit._Pos]
        mov     ecx, [esi+TEdit._Sel]
        cmp     eax, [esi+TEdit._Sel]
        je      .noselection            ; there is no selection.
        jg      @f
        xchg    eax, ecx        ; eax=end_selection; ecx=beg_selection
@@:
        add     [esi+TEdit._Len], ecx
        sub     [esi+TEdit._Len], eax
        mov     [esi+TEdit._Pos], ecx
        mov     [esi+TEdit._Sel], ecx

        push    -1
        stdcall StrOffsUtf8, [esi+TEdit._Text], eax
        push    eax
        stdcall StrOffsUtf8, [esi+TEdit._Text], ecx
        push    eax
        stdcall StrPtr, [esi+TEdit._Text]
        sub     [esp], eax     ; begin of the selection
        sub     [esp+4], eax
        mov     eax, [esp]
        sub     [esp+4], eax   ; length of the selection

        stdcall StrSplit, [esi+TEdit._Text]  ; position from the stack
        mov     ecx, eax
        stdcall StrCopyPart, ecx, ecx   ; pos and length from the stack.
        stdcall StrCat, [esi+TEdit._Text], ecx
        stdcall StrDel, ecx

        or      [.changes], .PosChanged or .NeedRefresh
        clc
        retn

.noselection:
        stc
        retn
endp


endmodule


;        stdcall DecodeUtf8, [edi]
;        jc      .finish
;
;        test    eax, eax
;        jz      .eraseremaining
;
;        mov     eax, [clEditTxt]
;        mov     esi, [clEditBk]
;        cmp     ecx, [.selbeg]
;        jl      .colorok
;        cmp     ecx, [.selend]
;        jge     .colorok
;        mov     eax, [clEditSelTxt]
;        mov     esi, [clEditSel]
;.colorok:
;        push    eax  0 [.ypos] [.bounds.x] edx edi
;
;        push    0 edx edi
;        add     edi, edx
;        stdcall GetTextBounds, [ebx+TPaintEvent.context] ; remaining from the stack
;
;        add     eax, 2
;        stdcall DrawFillRect, [ebx+TPaintEvent.context], [.bounds.x], [.bounds.y], eax, [.bounds.height], esi
;        sub     eax, 2
;
;        add     [.bounds.x], eax
;        sub     [.bounds.width], eax
;        jc      .nodraw
;
;        stdcall DrawString, [ebx+TPaintEvent.context] ; remaining from the stack
;
;        inc     ecx
;        jmp     .drawloop
;
;.nodraw:
;        add     esp, 24 ; 6*4
;        jmp     .finish
;
;.eraseremaining:
;        stdcall DrawFillRect, [ebx+TPaintEvent.context], [.bounds.x], [.bounds.y], [.bounds.width], [.bounds.height], [clEditBk]
;        jmp     .finish
;
;endp
;