; _______________________________________________________________________________________
;|                                                                                       |
;| ..::FreshLib::..  Free, open source. Licensed under "BSD 2-clause" license."          |
;|_______________________________________________________________________________________|
;
;  Description: TTreeView object class
;
;  Target OS: Any
;
;  Dependencies:
;  Notes:
;_________________________________________________________________________________________
module "TreeView library"

ObjectClass TreeView,                    \
            ScrollWindow,                \
            TTreeView.Create,           \
            TTreeView.Destroy,          \
            TTreeView.Get,              \
            TTreeView.Set,              \
            TTreeView.ExecCmd,          \
            TTreeView.SysEventHandler

; tree-view item state flags.
tvisExpanded = 1
tvisSelected = 2
tvisFocused  = 4


struct TTreeViewItem
  .level      dd ?        ; the level of the element in the tree.

  .caption    dd ?        ; hString of the name.
  .state      dd ?        ; one or more state flags.
  .imgNormal  dd ?        ; index to the icon for normal state
  .iSelected  dd ?        ; index to the icon for selected state

  .pObject    dd ?        ; the object attached to the item
  align 32
  .shift = 5
ends


object TTreeView, TScrollWindow
  ._items dd ?    ; TArray with
  ._first dd ?    ; index of the fisrt visible item.
  ._pShadow dd ?  ; pointer to TBackBuffer

  .OnItemDelete  dd ? ; procedure to be call when the item is deleted. It should free the objects attached to the item.

  param .Focused

  method .AddItem, .iParent
  method .DelItem, .iItem  ;deletes the item and all children.
  method .GetItem, .index
endobj


proc TTreeView.Create, .pobj
begin
        mov     ebx, [.pobj]

        stdcall CreateBackBuffer, [ebx+TTreeView.handle], 1, 1
        mov     [ebx+TTreeView._pShadow], eax

        stdcall CreateArray, sizeof.TTreeViewItem
        mov     [ebx+TTreeView._items], eax

        clc
        return
endp


proc TTreeView.Destroy, .pobj
begin
        mov     ebx, [.pobj]

        stdcall DestroyBackBuffer, [ebx+TTreeView._pShadow]
        execute ebx, TTreeView.DelItem, -1
        stdcall FreeMem, [ebx+TTreeView._items]

        clc
        return
endp


proc TTreeView.Get, .pobj, .paramID
begin
        stc
        return
endp


proc TTreeView.Set, .pobj, .paramID, .value
begin
        stc
        return
endp


proc TTreeView.ExecCmd, .self, .method
begin
        stc
        return
endp


proc TTreeView.SysEventHandler, .pobj, .pEvent
begin
        pushad

        mov     ebx, [.pEvent]
        mov     esi, [.pobj]
        mov     eax, [ebx+TSysEvent.event]

        cmp     eax, sePaint
        je      .paint

        cmp     eax, seMoveResize
        je      .moveresize

.continue:
        popad
        stc
        return

.paint:
        stdcall SetClipRectangle, [ebx+TPaintEvent.context], 0
        stdcall TScrollWindow.SysEventHandler, [.pobj], [.pEvent]
        stdcall DrawBackBuffer, [ebx+TPaintEvent.context], [esi+TTreeView._pShadow], 0, 0
        jmp     .continue

.moveresize:
        stdcall Get, esi, TTreeView.FreeArea

        stdcall DestroyBackBuffer, [esi+TTreeView._pShadow]
        stdcall CreateBackBuffer, [esi+TTreeView.handle], [esi+TTreeView._FreeArea.width], [esi+TTreeView._FreeArea.height]
        mov     [esi+TTreeView._pShadow], eax
        jmp     .continue

endp


endmodule