; _______________________________________________________________________________________
;|                                                                                       |
;| ..::FreshLib::..  Free, open source. Licensed under "BSD 2-clause" license."          |
;|_______________________________________________________________________________________|
;
;  Description: Process management library. Linux part
;
;  Target OS: Linux
;
;  Dependencies: Uses pthreads library for thread creation.
;
;  Notes:
;_________________________________________________________________________________________

uses libX11, pthreads, libc

; Terminate current process
; Returns: eax = 0
proc Terminate, .exit_code
begin
        mov     eax, sys_exit
        mov     ebx, [.exit_code]
        int     $80
        return
endp


proc TerminateAll, .exit_code
begin
        mov     eax, sys_exit_group
        mov     ebx, [.exit_code]
        int     $80
        return
endp


proc GetCmdArguments
begin
        push    ecx edx esi

        mov     esi, [_ptrEnvironment]

        mov     ecx, [esi]
        add     esi, 4

        stdcall CreateArray, 4
        mov     edx, eax
        jecxz   .array_ok

        stdcall AddArrayItems, eax, ecx

.array_ok:
        push    edx
        jecxz   .arg_ok

        lea     edx, [edx+TArray.array]

.loop:
        lodsd
        stdcall StrDup, eax
        mov     [edx], eax
        add     edx, 4
        loop    .loop

.arg_ok:
        pop     eax
        pop     esi edx ecx
        return
endp


match =LinuxThreads, LinuxThreads {
  define LinuxThreads posix
}

match =native, LinuxThreads {
  include '_process_native.asm'
}

match =posix, LinuxThreads {
  include '_process_posix.asm'
}




