; _______________________________________________________________________________________
;|                                                                                       |
;| ..::FreshLib::..  Free, open source. Licensed under "BSD 2-clause" license."          |
;|_______________________________________________________________________________________|
;
;  Description: OS dependent part of the network library.
;
;  Target OS: Win32
;
;  Dependencies:
;
;  Notes: This library follows the rules of Berkley sockets library with some small
;         differences (for example in the return values)
;_________________________________________________________________________________________

uses wsock32

uglobal
  __wsaData WSADATA
endg


if used SocketCreate

initialize InitWindowsSockets
begin
        invoke  WSAStartup, $0101, __wsaData
        test    eax, eax
        jz      @f
        int3
@@:
        return
endp

finalize FinishWindowsSockets
begin
        invoke  WSACleanup
        return
endp

end if


proc __SocketError
begin
        cmp     eax, SOCKET_ERROR
        jne     .no_error

        push    ecx edx
        invoke  WSAGetLastError
        pop     edx ecx
        stc
        return

.no_error:
        clc
        return
endp


proc SocketCreate, .protocol_family, .socket_type, .protocol
begin
        push    ecx edx
        invoke  socket, [.protocol_family], [.socket_type], [.protocol]
        call    __SocketError
        pop     edx ecx
        return
endp



proc SocketClose, .hSocket
.buffer rb 1024
begin
        push    ecx edx
        invoke  shutdown, [.hSocket], 1
        call    __SocketError
        jc      .finish

.loop:
        lea     ecx, [.buffer]
        invoke  recv, [.hSocket], ecx, 1024, 0
        call    __SocketError
        jc      .finish
        test    eax, eax
        jnz     .loop

        invoke  closesocket, [.hSocket]
        call    __SocketError

.finish:
        pop     edx ecx
        return
endp



proc SocketConnect, .hSocket, .pAddress
begin
        push    ecx edx
        invoke  connect, [.hSocket], [.pAddress], sizeof.TSocketAddress
        call    __SocketError
        pop     edx ecx
        return
endp



proc SocketBind, .hSocket, .pAddressIn
begin
        push    ecx edx
        invoke  bind, [.hSocket], [.pAddressIn], sizeof.TSocketAddressIn
        call    __SocketError
        pop     edx ecx
        return
endp



proc SocketListen, .hSocket, .maxPending
begin
        push    ecx edx
        invoke  listen, [.hSocket], [.maxPending]
        call    __SocketError
        pop     edx ecx
        return
endp



proc SocketAccept, .hSocket, .pAddress
.addrlen dd ?
begin
        push    edx

        mov     [.addrlen], sizeof.TSocketAddress
        lea     ecx, [.addrlen]
        cmp     [.pAddress], 0
        jne     @f
        xor     ecx, ecx
        mov     [.addrlen], ecx
@@:
        invoke  accept, [.hSocket], [.pAddress], ecx
        mov     ecx, [.addrlen]
        call    __SocketError
        pop     edx
        return
endp



proc SocketSend, .hSocket, .pBuffer, .DataLen, .flags
begin
        push    ecx edx
        invoke  send, [.hSocket], [.pBuffer], [.DataLen], [.flags]
        call    __SocketError
        pop     edx ecx
        return
endp


proc SocketReceive, .hSocket, .pBuffer, .BufferSize, .flags
begin
        push    ecx edx
        invoke  recv, [.hSocket], [.pBuffer], [.BufferSize], [.flags]
        call    __SocketError
        pop     edx ecx
        return
endp


proc SocketSendTo, .hSocket, .pBuffer, .DataLen, .flags, .pAddressTo
begin
        push    ecx edx
        invoke  sendto, [.hSocket], [.pBuffer], [.DataLen], [.flags], [.pAddressTo], sizeof.TSocketAddress
        call    __SocketError
        pop     edx ecx
        return
endp


proc SocketReceiveFrom, .hSocket, .pBuffer, .BufferSize, .flags, .pAddressFrom
.addrlen dd ?
begin
        push    edx
        mov     [.addrlen], sizeof.SocketAddress
        lea     ecx,  [.addrlen]
        invoke  recvfrom, [.hSocket], [.pBuffer], [.BufferSize], [.flags], [.pAddressFrom], ecx
        call    __SocketError
        mov     ecx, [.addrlen]
        pop     edx
        return
endp


;proc SetLinger, .hSocket, .fLinger, .timeout
;begin
;        push    ecx edx
;        lea     ecx, [.fLinger]
;        invoke  setsockopt, [.hSocket], SOL_SOCKET, SO_LINGER, ecx, 8
;        call    __SocketError
;        pop     edx ecx
;        return
;endp