; _______________________________________________________________________________________
;|                                                                                       |
;| ..::FreshLib::..  Free, open source. Licensed under "BSD 2-clause" license."          |
;|_______________________________________________________________________________________|
;
;  Description: Process management library. Win32 part.
;
;  Target OS: Win32
;
;  Dependencies:
;
;  Notes:
;_________________________________________________________________________________________

uses kernel32, shell32

struct TMutex
  .handle dd ?
ends

proc Terminate, .exit_code
begin
        invoke ExitThread, [.exit_code]
        return
endp


; Terminate current process
; Returns: nothing.

proc TerminateAll, .exit_code
begin
        invoke ExitProcess, [.exit_code]
        return
endp



proc GetCmdArguments
.argc dd ?
.array dd ?
begin
        pushad
        invoke  GetCommandLineW
        lea     ecx, [.argc]
        invoke  CommandLineToArgvW, eax, ecx
        test    eax, eax
        jz      .error

        push    eax
        mov     esi, eax

        stdcall CreateArray, 4
        mov     [.array], eax

.argloop:
        cmp     [.argc], 0
        je      .endarg

        stdcall WideCharToUtf8, [esi]
        push    eax
        stdcall AddArrayItems, [.array], 1
        mov     [.array], edx
        pop     dword [eax]

        add     esi, 4
        dec     [.argc]
        jmp     .argloop

.endarg:
        invoke  LocalFree ; from the stack...
        popad
        mov     eax, [.array]
        clc
        return

.error:
        popad
        stc
        return
endp



; Creates a new execution thread
; Returns: CF=1 if error.
;          CF=0, eax = pid of the new process being created

proc ThreadCreate, .ptr_to_function, .ptr_to_args
begin
        push    ecx edx

        invoke  CreateThread, NULL, 0, [.ptr_to_function], [.ptr_to_args], 0, 0
        test    eax, eax
        jz      .error

        clc
        pop     edx ecx
        return

.error:
        stc
        pop     edx ecx
        return
endp



proc FreeThreadID, .ThreadID
begin
        push    eax ecx edx
        invoke  CloseHandle, [.ThreadID]
        pop     edx ecx eax
        return
endp



; MUTEX functions
;_________________________________________________________________________________________

proc MutexCreate, .ptrName, .ptrMutex
begin
        push    eax ecx edx

        invoke  CreateMutexW, 0, TRUE, 0
        mov     ecx, [.ptrMutex]
        mov     [ecx+TMutex.handle], eax
        pop     edx ecx eax
        return
endp




proc WaitForMutex, .ptrMutex, .timeout
begin
        push    eax ecx edx
        mov     ecx, [.ptrMutex]
        invoke  WaitForSingleObject, [ecx+TMutex.handle], [.timeout]
        cmp     eax, WAIT_TIMEOUT
        je      .error
        cmp     eax, WAIT_FAILED
        je      .error

        clc
.finish:
        pop     edx ecx eax
        return

.error:
        int3
        stc
        jmp     .finish
endp




proc MutexRelease, .ptrMutex
begin
        pushf
        push    eax ecx edx

        mov     eax, [.ptrMutex]
        invoke  ReleaseMutex, [eax+TMutex.handle]

        pop     edx ecx eax
        popf
        return
endp



proc MutexDestroy, .ptrMutex
begin
        push    eax ecx edx
        mov     eax, [.ptrMutex]
        pushd   [eax]
        mov     dword [eax], 0
        invoke  CloseHandle ; from the stack
        pop     edx ecx eax
        return
endp

