; This module is responsible for creating/reading/saving of common for
; the whole IDE parameters from the main .ini file.
;

cParamSection text 'IDE settings'
cDebugerKey   text 'Debugger'
cHelpSection  text 'HelpFiles'

cAutoHideKey  text 'AutoHideEnable'
cAutoHideTime text 'AutoHideTime'

cErrStr2      text 'ERR!'


uglobal
  flagAutoHide dd ?
  timeAutoHide dd ?
  blockAutoHide dd ?
endg



proc CheckParams
begin
        invoke  SendMessageA, [frmMsg], MWM_CLEARMESSAGES, 0, 0

; Check ini file consistency...
        stdcall StrPtr, [hIniFileName]
        mov     esi, eax

        stdcall FileExists, esi
        jnc     .doCheckParams  ; if the file exists, go check params.

        stdcall FirstTimeInstall       ; try to create .ini file automatically

.doCheckParams:
        stdcall CheckFilesSection, cHelpSection, cHelpFileNotSet, cHelpFileNotExists
        stdcall CheckFileParam, cDebugerKey, cDebuggerNotSet, cDebuggerNotExists

; Load IDE options from the .ini file
        invoke  GetPrivateProfileIntA, cParamSection, cAutoHideKey, 0, esi
        mov     [flagAutoHide], eax
        invoke  GetPrivateProfileIntA, cParamSection, cAutoHideTime, 5000, esi
        mov     [timeAutoHide], eax

        stdcall RebuildHelpFilesList

        call SetAutoHideTimer
        return

  cHelpFileNotExists text  "Some of configured help files doesn't exists."
  cHelpFileNotSet    text  "Not any help file is set in IDE options.",0

  cDebuggerNotExists text  "Specified external debugger doesn't exists."
  cDebuggerNotSet    text  "No external debugger is set in IDE options."
endp



proc CheckFileParam, .ptrKeyName, .msgNotSet, .msgNotExists
begin
        stdcall GetParamFromINI, [.ptrKeyName]
        jc      .filenotset

        push    eax
        stdcall StrPtr, eax
        stdcall FileExists, eax
        jc      .notexists

        stdcall StrDel ; from the stack
        return

.notexists:
        stdcall StrDel ; from the stack
        stdcall AddCheckMessage, [.msgNotExists], mtError
        return

.filenotset:
        stdcall AddCheckMessage, [.msgNotSet], mtWarning
        return
endp


proc CheckFilesSection, .ptrSectionName, .msgNotSet, .msgNotExists
.count dd ?
.bad   dd ?
begin
        mov     [.count], 0
        mov     [.bad], 0
        lea     eax, [.count]
        stdcall ReadSectionFromIni, [hIniFileName], [.ptrSectionName], _OnCheckOneFile, eax

        cmp     [.count], 0
        je      .filenotset

        cmp     [.bad], 0
        jne     .notexists

.finish:
        return

.notexists:
        stdcall AddCheckMessage, [.msgNotExists], mtError
        return

.filenotset:
        stdcall AddCheckMessage, [.msgNotSet], mtWarning
        return
endp


proc _OnCheckOneFile, .ptrFileName, .ptrCounts
begin
        mov     esi, [.ptrCounts]
        inc     dword [esi]

        stdcall FileExists, [.ptrFileName]
        jnc     .finish

        inc     dword [esi+4]

.finish:
        clc
        return
endp



;-------------------------------------
; It tries to create proper ini file
; in the case that ini file does not
; exists.
;-------------------------------------
proc FirstTimeInstall
.str rb 1024
.filename dd ?
begin
        stdcall SearchTablePath, cPathsTableInc, _section_environment, cIncludeKey, 1, cNotFoundMsg1, esi
        stdcall SearchTablePath, cPathsTableMain, _section_environment, cFreshKey, 1, cNotFoundMsg2, esi
        stdcall SearchTablePath, cPathsTableLib, _section_environment, cLibKey, 1, cNotFoundMsg3, esi

        stdcall SearchTablePath, cFreshGuideTable, cHelpSection, help_first_key, 0, cErrorFreshGuide, esi
        return
endp

iglobal
  cFreshGuideTable dd  .guide1, .guide2, 0
  .guide1 db '../'
  .guide2 db 'doc/FreshGuide.chm', 0

  help_first_key db 'aaa', 0

  cErrorFreshGuide db 'FreshGuide.chm not found.'
endg





proc SearchTablePath, .ptrTable, .ptrSection, .ptrKey, .fPathOnly, .ptrErrorMsg, .ptrIniFile
  .str rb 1024
  .ptr dd ?
begin
        pushad

        mov     ebx, [.ptrTable]

.findloop:
        mov     ecx, [ebx]
        test    ecx, ecx
        jz      .notfound

        lea     edi, [.str]
        lea     eax, [.ptr]
        invoke  GetFullPathNameA, ecx, 1024, edi, eax
        stdcall FileExists, edi
        lea     ebx, [ebx+4]
        jc      .findloop

        cmp     [.fPathOnly], 0
        je      @f
        mov     eax, [.ptr]
        mov     byte [eax-1], 0

@@:
        invoke  WritePrivateProfileStringA, [.ptrSection], [.ptrKey], edi, [.ptrIniFile]

        clc
        popad
        return

.notfound:
        invoke  MessageBoxA, [hApplication], [.ptrErrorMsg], cNotFoundTtl, MB_ICONWARNING or MB_OK
        stc
        popad
        return
endp


iglobal
  cPathsTableInc dd cAutoPath1, cAutoPath2, 0
  cAutoPath2  db '..\'
  cAutoPath1  db 'include'
  cFincTestFile db '\finc.txt',0

  cPathsTableMain dd cAutoPath4, cAutoPath3, 0
  cAutoPath3  db '..\'
  cAutoPath4  db 'Fresh.exe',0

  cPathsTableLib dd cAutoPath6, cAutoPath5, 0
  cAutoPath5  db '..\'
  cAutoPath6  db 'freshlib'
  cLibTestFile db '\freshlib.inc',0
endg

cIncludeKey   text 'finc'
cFreshKey     text 'Fresh'
cLibKey       text 'lib'

cNotFoundMsg1 text "First time run procedure can't find include path automanically. Try to set it manualy in 'options|IDE params' menu.",     \
                     "You have to use 'finc' name for this path."
cNotFoundMsg2 text "First time run procedure can't find main program directory. Try to set it manually in 'options|IDE params' menu.",        \
                     "You have to use 'Fresh' name for this path.",0
cNotFoundMsg3 text "First time run procedure can't find FreshLib directory. Try to set it manually in 'options|IDE params' menu.",        \
                     "You have to use 'Lib' name for this path.",0
cNotFoundTtl  text 'Auto setup error'


proc AddCheckMessage, .ptrMessage, .msgType
.msg TFreshMsg
begin
        mov     eax, [.msgType]
        mov     ecx, [.ptrMessage]
        mov     [.msg.type], eax
        mov     [.msg.hMessage], ecx
        mov     [.msg.link.hFileName], 0

        lea     eax, [.msg]
        invoke  SendMessageA, [frmMsg], MWM_ADDMESSAGE, eax, TVI_ROOT
        return
endp




;------------------------------------------------
; Parces command line and opens the files.
;------------------------------------------------
proc ParseCommandLine
.num dd ?
begin
        pushad

        stdcall GetCmdArguments
        mov     esi, eax

        cmp     [esi+TArray.count], 2
        jb      .finish

        xor     ebx, ebx
        inc     ebx

.open_loop:
        stdcall OpenFileByName, [esi+TArray.array+4*ebx], NULL, TRUE
        inc     ebx
        cmp     ebx, [esi+TArray.count]
        jne     .open_loop

.finish:
        stdcall ListFree, esi, StrDel
        popad
        return
endp



proc GetParamFromINI, .ptrKeyName
.buff rb 1024
begin
        stdcall StrPtr, [hIniFileName]

        lea     ecx, [.buff]
        invoke  GetPrivateProfileStringA, cParamSection, [.ptrKeyName], cErrStr2, ecx, 1024, eax
        cmp     dword [.buff], 'ERR!'
        je      .error

        lea     eax, [.buff]
        stdcall StrDup, eax
        clc
        return

.error:
        xor     eax, eax
        stc
        return
endp





proc ReadSectionFromIni, .hIniFile, .hSectionName, .procCallback, .lparam
.ptrmem dd ?
begin
        push    ebx esi edi

        stdcall GetMem, 65536     ; should be enough
        mov     esi, eax
        mov     [.ptrmem], eax

        stdcall StrPtr, [.hIniFile]
        push    eax
        push    65536
        push    esi
        stdcall StrPtr, [.hSectionName]
        push    eax
        invoke  GetPrivateProfileSectionA ; parameters from the stack.

.addloop:
        cmp     byte [esi], 0
        je      .endofread

        pushad
        add     esi, 4  ; skip the key name
        stdcall [.procCallback], esi, [.lparam]
        popad
        jc      .endofread

        stdcall StrLen, esi
        lea     esi, [esi+eax+1]
        jmp     .addloop

.endofread:
        pushf
        stdcall FreeMem, [.ptrmem]
        popf
        pop     edi esi ebx
        return
endp